import java.util.EnumSet;
import java.util.List;

import com.mindfusion.charting.*;
import com.mindfusion.common.DateTime;


class MySeries implements Series
{
	/*
	 * All MySeries instances displayed by the chart contain a reference
	 * to same list of StockPrice objects, so we can avoid copying data.
	 */
	private List<StockPrice> prices;

	/*
	 * Indicates which of the StockPrice members is returned as data value.
	 */
	private PriceType priceType;

	/*
	 * The only constructor we allow must receive a reference to StockPrice list.
	 */
	public MySeries(List<StockPrice> prices, PriceType priceType)
	{
		this.prices = prices;
		this.priceType = priceType;
	}

	/*
	 * This lets the chart know how many data points it should draw.
	 */
	@Override
	public int getSize()
	{
		return prices.size();
	}

	/*
	 * For the BarChart in this example, we only return one-dimensional data (Y values).
	 */
	@Override
	public int getDimensions()
	{
		return 1;
	}

	/*
	 * Returns the data value by reading specified StockPrice member.
	 */
	@Override
	public double getValue(int index, int dimension)
	{
		switch (priceType)
		{
			case open:
				return prices.get(index).getOpen();
			case close:
				return prices.get(index).getClose();
			case low:
				return prices.get(index).getLow();
			case high:
				return prices.get(index).getHigh();
		}
		return 0;
	}

	/*
	 * We only show date string on the X axis for one of the series.
	 */
	@Override
	public EnumSet<LabelKinds> getSupportedLabels()
	{
		return priceType == PriceType.open ?
			EnumSet.of(LabelKinds.XAxisLabel) :
			EnumSet.noneOf(LabelKinds.class);
	}

	/*
	 * Can return data labels to draw inside or above bars, at either axis, or as tooltips.
	 */
	@Override
	public String getLabel(int index, LabelKinds kind)
	{
		DateTime date = new DateTime(
			prices.get(index).getDate()); 
		return date.toShortDateString();
	}

	/*
	 * Title strings are displayed inside the legend box.
	 */
	@Override
	public String getTitle()
	{
		return priceType.toString();
	}

	/*
	 * The charts draw respective data element as highlighted if this returns true.
	 */
	@Override
	public boolean isEmphasized(int index)
	{
		return false;
	}

	/*
	 * The chart can do some optimizations, such as binary search for viewport
	 * clipping, if it knows data values are returned sorted.
	 * Return true if data is actually sorted, or you will get incorrect results.
	 */
	@Override
	public boolean isSorted(int arg0)
	{
		return false;
	}

	/*
	 * The chart refreshes automatically if this event is raised (not used in this example).
	 */
	@Override
	public void addDataChangedListener(DataChangedListener listener) {}
	@Override
	public void removeDataChangedListener(DataChangedListener listener) {}
}

enum PriceType
{
	open,
	close,
	low,
	high
};
