import java.awt.*;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import java.util.stream.Collectors;

import javax.swing.*;

import com.mindfusion.drawing.*;
import com.mindfusion.charting.*;
import com.mindfusion.charting.components.LayoutAlignment;
import com.mindfusion.charting.swing.*;
import com.mindfusion.common.ObservableList;
import com.mindfusion.common.SwingExecutor;


public class Tutorial3
{
	Chart createChart()
	{
		BarChart chart = new BarChart();
		
		fetchStockPrices("ACME").thenAcceptAsync(
			(stockPrices) ->
			{
				// a series of daily opening prices
				chart.getSeries().add(
				    new MySeries(stockPrices, PriceType.open));
	
				// a series of daily closing prices
				chart.getSeries().add(
				    new MySeries(stockPrices, PriceType.close));
	
				// display min and max prices as a line chart
				ObservableList<Series> minMaxSeries = new ObservableList<>();
				minMaxSeries.add(new MySeries(stockPrices, PriceType.low));
				minMaxSeries.add(new MySeries(stockPrices, PriceType.high));
	
				LineRenderer minMaxRenderer = new LineRenderer(minMaxSeries);
				PerSeriesStyle minMaxStyle = new PerSeriesStyle();
				minMaxStyle.setStrokes(
					Colors.Red, Colors.Green);
				minMaxStyle.setFills(
					Colors.Red, Colors.Green);
				minMaxStyle.setStrokeThicknesses(List.of(4.0));
				minMaxRenderer.setSeriesStyle(minMaxStyle); 
	
				chart.getPlot().getSeriesRenderers().add(minMaxRenderer);
				chart.getLegendRenderer().getContent().add(minMaxRenderer);
	
			},
			new SwingExecutor());

		// assign different colors to bar series
		chart.getTheme().setCommonSeriesFills(
			Colors.Yellow, Colors.LightBlue);

		// customize axis appearance
		chart.setTitle("ACME stock price weekly");
		chart.getYAxis().setMinValue(0.0);
		chart.getYAxis().setTitle("stock price in USD");
		chart.getXAxis().setTitle("");
		chart.setShowXCoordinates(false);
		chart.setShowXTicks(false);
		chart.setLegendHorizontalAlignment(LayoutAlignment.Far);

		return chart;
	}

	static CompletableFuture<List<StockPrice>> fetchStockPrices(String name)
	{
		// pretend we are calling a web service, returning
		// stock price statistics for last 7 days
        return CompletableFuture.supplyAsync(() ->
        {
			var random = new Random();
			Instant date = Instant.now().minus(8, ChronoUnit.DAYS);

			var prices = new ArrayList<StockPrice>();
			for (int i = 0; i < 7; i++)
			{
				date = date.plus(1, ChronoUnit.DAYS);

				double open = 200 + 500 * random.nextDouble();
				double close = open + 50 * (0.5 - random.nextDouble());
				double low = Math.min(open, close) - 50 * random.nextDouble();
				double high = Math.max(open, close) + 50 * random.nextDouble();

				prices.add(
					new StockPrice(open, close, low, high, Date.from(date)));
			}
			return prices;
        });
    }

	static <R, T> List<R> mapList(List<T> list, Function<T, R> mapper)
	{
		return list.stream().map(mapper).collect(Collectors.toList());
	}

	Tutorial3()
	{
		JFrame frame = new JFrame();
		frame.setTitle("MindFusion.Charting sample: Tutorial 3");
		frame.setSize(1000, 600);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// add chart component to the middle of the frame
		frame.getContentPane().add(
			createChart(), BorderLayout.CENTER);

		JTextArea description = new JTextArea(
			"This example demonstrates how to add different types of chart graphics to the plot.");
		description.setEditable(false);
		frame.getContentPane().add(
			description, BorderLayout.SOUTH);

		frame.setVisible(true);
	}

	static public void main(String[] args)
	{
		SwingUtilities.invokeLater(Tutorial3::new);
	}
}
